@@ -1,13 +1,41 @@
+/*
+ * Bognor-Regis - a media player/queue daemon.
+ * Copyright © 2009, Intel Corporation.
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms and conditions of the GNU Lesser General Public License,
+ * version 2.1, as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ * more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA
+ */
+
+#include "config.h"
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-bindings.h>
 
+#ifdef USE_QUEUE_CONFIG
+#include <gconf/gconf-client.h>
+#endif
+
 #include "br-marshal.h"
 #include "br-queue.h"
 #include "bognor-queue-bindings.h"
 
+/* default settings */
+#define BR_QUEUE_DBUS_SERVICE "org.moblin.BognorRegis"
+#define BR_LOCAL_QUEUE_PATH "/org/moblin/BognorRegis/Queues/local_queue"
+
 enum {
     PROP_0,
-    PROP_PATH
+    PROP_PATH,
+    PROP_SERVICE,
 };
 
 enum {
@@ -22,6 +50,7 @@ enum {
 struct _BrQueuePrivate {
     DBusGProxy *proxy;
     char *object_path;
+    char *service_name;
 };
 
 struct _AsyncClosure {
@@ -65,7 +94,11 @@ br_queue_finalize (GObject *object)
         priv->object_path = NULL;
     }
 
-    g_signal_handlers_destroy (object);
+    if (priv->service_name) {
+        g_free (priv->service_name);
+        priv->service_name = NULL;
+    }
+
     G_OBJECT_CLASS (br_queue_parent_class)->finalize (object);
 }
 
@@ -109,6 +142,10 @@ br_queue_set_property (GObject      *object,
         priv->object_path = g_value_dup_string (value);
         break;
 
+    case PROP_SERVICE:
+        priv->service_name = g_value_dup_string (value);
+        break;
+
     default:
         break;
     }
@@ -170,7 +207,6 @@ playing_changed (DBusGProxy *proxy,
                  gboolean    playing,
                  BrQueue    *queue)
 {
-    g_print ("Emitting playing changed\n");
     g_signal_emit (queue, signals[PLAYING_CHANGED], 0, playing);
 }
 
@@ -205,20 +241,20 @@ br_queue_constructor (GType                  type,
                                        DBUS_PATH_DBUS,
                                        DBUS_INTERFACE_DBUS);
     if (!org_freedesktop_DBus_start_service_by_name (proxy,
-                                                     BR_QUEUE_DBUS_SERVICE, 0,
+                                                     priv->service_name, 0,
                                                      &start_ret, &error)) {
-        g_warning ("bognor-regis-daemon could not be started: %s",
-                   error->message);
+        g_warning ("bognor-regis could not start queue service '%s': %s",
+		   priv->service_name, error->message);
         g_error_free (error);
 
         priv->proxy = NULL;
         return G_OBJECT (queue);
     }
 
-    priv->proxy = dbus_g_proxy_new_for_name (connection,
-                                             BR_QUEUE_DBUS_SERVICE,
+    priv->proxy = dbus_g_proxy_new_for_name (connection, priv->service_name,
                                              priv->object_path,
                                              BR_QUEUE_DBUS_INTERFACE);
+
     dbus_g_proxy_add_signal (priv->proxy, "UriAdded",
                              G_TYPE_STRING, G_TYPE_INT, G_TYPE_INVALID);
     dbus_g_proxy_add_signal (priv->proxy, "UriRemoved",
@@ -268,6 +304,13 @@ br_queue_class_init (BrQueueClass *klass)
                                                           G_PARAM_WRITABLE |
                                                           G_PARAM_CONSTRUCT_ONLY |
                                                           G_PARAM_STATIC_STRINGS));
+    g_object_class_install_property (o_class, PROP_SERVICE,
+                                     g_param_spec_string ("service-name",
+                                                          "", "", "",
+                                                          G_PARAM_WRITABLE |
+                                                          G_PARAM_CONSTRUCT_ONLY |
+                                                          G_PARAM_STATIC_STRINGS));
+
     signals[ADDED] = g_signal_new ("uri-added",
                                    G_TYPE_FROM_CLASS (klass),
                                    G_SIGNAL_RUN_FIRST |
@@ -312,6 +355,75 @@ br_queue_init (BrQueue *self)
     self->priv = GET_PRIVATE (self);
 }
 
+BrQueue *
+br_queue_new (const char *service_name,
+              const char *object_path)
+{
+    BrQueue *queue;
+
+    queue = g_object_new (BR_TYPE_QUEUE,
+                          "object-path", object_path,
+                          "service-name", service_name,
+                          NULL);
+    return queue;
+}
+
+/* allow different implementations of the queue service */
+
+static char *
+get_queue_dbus_service (void)
+{
+#ifdef USE_QUEUE_CONFIG
+    char *queue;
+
+    queue = gconf_client_get_string
+        (gconf_client_get_default (),
+	 "/desktop/moblin/media_queue/service", NULL);
+    if (queue)
+        return queue;
+#endif
+
+    return g_strdup (BR_QUEUE_DBUS_SERVICE);
+}
+
+static char *
+get_local_dbus_path (void)
+{
+#ifdef USE_QUEUE_CONFIG
+    char *path;
+
+    path = gconf_client_get_string
+        (gconf_client_get_default (),
+	 "/desktop/moblin/media_queue/local_queue", NULL);
+    if (path)
+        return path;
+#endif
+
+    return g_strdup (BR_LOCAL_QUEUE_PATH);
+}
+
+BrQueue *
+br_queue_new_local (void)
+{
+    BrQueue *queue;
+
+#ifdef USE_QUEUE_CONFIG
+    char *path, *service;
+
+    path = get_local_dbus_path ();
+    service = get_queue_dbus_service ();
+
+    queue = br_queue_new (service, path);
+
+    g_free (path);
+    g_free (service);
+#else
+    queue = br_queue_new (BR_QUEUE_DBUS_SERVICE, BR_LOCAL_QUEUE_PATH);
+#endif
+
+    return queue;
+}
+
 static void
 async_reply (DBusGProxy *proxy,
              GError     *error,
@@ -653,3 +765,4 @@ br_queue_get_playing (BrQueue                  *queue,
                                                     get_playing_reply,
                                                     data);
 }
+
