@@ -1,3 +1,21 @@
+/*
+ * Bognor-Regis - a media player/queue daemon.
+ * Copyright © 2009, Intel Corporation.
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms and conditions of the GNU Lesser General Public License,
+ * version 2.1, as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ * more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA
+ */
+
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <gst/gst.h>
@@ -118,26 +136,23 @@ should_show_audio_as_visual (BognorQueue *queue)
 {
     BognorLocalQueue *local = (BognorLocalQueue *) queue;
     BognorLocalQueuePrivate *priv = local->priv;
+    BognorQueueItem *item = bognor_queue_get_current_item (queue);
     GError *error = NULL;
 
     if (priv->hornsey_running == FALSE) {
         return;
     }
 
-    if (bognor_queue_get_visual_enabled (queue)) {
-        BognorQueueItem *item = bognor_queue_get_current_item (queue);
+    if (item == NULL || item->type == VISUAL_TYPE) {
+        return;
+    }
 
-        g_print ("Showing audio as visual\n");
-        if (item == NULL) {
-            return;
-        }
-
-        if (!org_moblin_BognorRegis_Player_show_uri (priv->hornsey, item->uri,
-                                                     item->mimetype, &error)) {
-            g_warning ("Error telling Hornsey about %s: %s", item->uri,
-                       error->message);
-            g_error_free (error);
-        }
+    g_print ("Showing audio as visual\n");
+    if (!org_moblin_BognorRegis_Player_show_uri (priv->hornsey, item->uri,
+                                                 item->mimetype, &error)) {
+        g_warning ("Error telling Hornsey about %s: %s", item->uri,
+                   error->message);
+        g_error_free (error);
     }
 }
 
@@ -155,6 +170,11 @@ set_audio_uri (BognorQueue     *queue,
         g_object_set (priv->playbin,
                       "uri", "",
                       NULL);
+
+        if (priv->audio_state == GST_STATE_PLAYING) {
+            gst_element_set_state (priv->playbin, GST_STATE_PAUSED);
+            priv->audio_state = GST_STATE_PAUSED;
+        }
         return;
     }
 
@@ -519,6 +539,9 @@ can_show_visual_changed (DBusGProxy  *proxy,
                          BognorQueue *queue)
 {
     bognor_queue_set_visual_enabled (queue, can_show);
+    if (can_show) {
+        should_show_audio_as_visual (queue);
+    }
 }
 
 static void
